# This file is NOT auto-generated

"""
Python modules and scripts for working with
[Concrete](https://github.com/hltcoe/concrete), an HLT data
specification defined using [Thrift](http://thrift.apache.org).
"""


from audio.ttypes import *
from clustering.ttypes import *
from communication.ttypes import *
from communication_service.ttypes import *
from discourse.ttypes import *
from email.ttypes import *
from entities.ttypes import *
from language.ttypes import *
from linking.ttypes import *
from metadata.ttypes import *
from nitf.ttypes import *
from situations.ttypes import *
from spans.ttypes import *
from structure.ttypes import *
from twitter.ttypes import *
from uuid.ttypes import *

from version import __version__
